
/**
 * RmiJdbc client/server JDBC Driver
 * (C) GIE Dyade (Groupe BULL / INRIA Research Center) 1997
 *
 * @version     1.0
 * @author      Pierre-Yves Gibello (Pierre-Yves.Gibello@inrialpes.fr)
 */

package RmiJdbc;

import java.sql.DriverManager;
import java.sql.Driver;
import java.sql.Connection;
import java.rmi.*;
import java.rmi.registry.*;
import java.net.InetAddress;
import java.util.Vector;
import java.util.Hashtable;

import RmiJdbc.RJDriverServer;

/**
 * The main class for RMI/JDBC Server
 */
public class RJJdbcServer {

  static Vector drivers_ = new Vector();
  static int port_ = -1;
  static boolean startreg_ = true;

  // GC problem on Linux Red Hat 6.0 + JDK v1.1.7B (Blackdown)
  // Thanks to Per Widerlund from Parallel Systems for the fix.
  static DriverManager dummy_for_gc;

  /**
   * Process the command-line parameters
   */
  static void processArgs(String args[]) {

    int nb = args.length;

    Hashtable hash = new Hashtable();
    hash.put("-noreg",new Integer(0));
    hash.put("-port",new Integer(1));
    hash.put("-?",new Integer(2));
    hash.put("?",new Integer(2));
    hash.put("-h",new Integer(2));

    for(int argnum=0; argnum < nb; argnum++) {

      Integer opt = (Integer)hash.get(args[argnum]);
      if(opt == null) opt = new Integer(-1);

      switch (opt.intValue()) {

       case 0:
        startreg_ = false;
        break;

       case 1:
        try {
          port_ = Integer.parseInt(args[++argnum]);
        } catch(Exception e) {
          System.err.println("Error: port must be a number");
          System.exit(1);
        }
        break;

       case 2:
        RJJdbcServer.printUsage();
        System.exit(1);

       default:
        drivers_.addElement(args[argnum]);
        break;
      }
    }
  }

  static void printUsage() {
    System.out.println(
     "Usage:\tjava RmiJdbc.RJJdbcServer [-noreg] [-port port] [driver]*");
    System.out.println(
     "-noreg: No internal registry, requires rmiregistry to be started");
    System.out.println(
     "-port: specify a port number for the rmi registry (default: 1099)");
    System.out.println("[driver]*: A list of JDBC driver classes"); 
  }

  
  void register(String name, int port, boolean startreg)
  throws Exception {

    String host;
    try {
      host = System.getProperty("java.rmi.server.hostname");
    } catch (Exception e) {
      host = null; 
      System.out.println("WARNING: java.rmi.server.hostname property"
       + " can\'t be read (access denied)");
      System.out.println("If you use java.rmi.server.hostname, set the"
       + " corresponding property to \"read\" in your java.policy file");
    }
    if(host == null) host = InetAddress.getLocalHost().getHostName();

    String rmiRef = "//" + host + "/" + name;

    if(port > 0) {
      rmiRef = new String("//" + host + ":" + port + "/" + name);
    }
	
	// RWS REMOVED
    // RJDriverServer theDriver = new RJDriverServer();
    RJDriverServer theDriver = buildDriverServer(); // RWS ADDED

    if(! startreg) { // External registry assumed
      Naming.rebind(rmiRef, theDriver);
      return;
    }

    // No external registry, start one
    if(port <= 0) port = 1099;
    Registry registry = LocateRegistry.createRegistry(port);
    registry.rebind(name, theDriver);
//    registry.rebind(rmiRef, theDriver);
  }
  
  
  /**
   * Build the driver server object. This is a separate method
   * so extensions can build different driver-server classes.
   * 
   * @since 8-May-1999
   */
  RJDriverServer buildDriverServer()
  throws java.rmi.RemoteException
  {
    return new RJDriverServer();
  }

  public static void main(String[] args) {

    processArgs(args);

    // RWS ADDED -- broke up main into two routines
    initServer(new RJJdbcServer()); // RWS ADDED
  } // RWS ADDED
  
  static void initServer(RJJdbcServer theServer) // RWS ADDED
  { // RWS ADDED
    try {

      // Args on the command line are interpreted as jdbc Driver class names
      // Try to register them in the jdbc DriverManager
      // For example, to register the driver for InstantDB free database:
      // java RJJdbcServer jdbc.idbDriver
      // Of course, you can also use the jdbc.drivers System property

      for(int i = 0; i < drivers_.size(); i++) {
        String drv = (String)drivers_.elementAt(i);
        try {
          // Class.forName(...) should be enough, without newInstance() !
          // On some platforms, it is not...
          Class.forName(drv).newInstance();
          System.out.println(drv + " registered in DriverManager");
        } catch(Exception e) {
          System.err.println("*** Can't register jdbc Driver for " + drv);
          System.err.println("Error message is: " + e.getMessage());
        }
      }

      // The following code is useful on some platforms where the ClassLoader
      // has a strange behaviour (that causes "No suitable driver" exceptions
      // concerning properly registered drivers !)
      java.util.Enumeration ed = DriverManager.getDrivers();
      while(ed.hasMoreElements()) { Driver d = (Driver)ed.nextElement(); }

      System.out.println("Binding RmiJdbcServer...");
      System.setSecurityManager(new RMISecurityManager());
      
      // RWS REMOVED
      // RJJdbcServer theServer = new RJJdbcServer();

      theServer.register("RmiJdbcServer", port_, startreg_);
      System.out.println("RmiJdbcServer bound in rmi registry");

      // If the server has its own registry, make sure the process keeps running
      if(startreg_) {
        // suspend the main thread to ensure that this process
        // will continue to run
//      Thread.currentThread().suspend();
        Thread tt = new Thread();
        tt.suspend();
      }

    } catch(Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
  }
  
  

};

