
/**
 * RmiJdbc client/server JDBC Driver
 * (C) GIE Dyade (Groupe BULL / INRIA Research Center) 1997
 *
 * @version     1.0
 * @author      Pierre-Yves Gibello (Pierre-Yves.Gibello@inrialpes.fr)
 */

package RmiJdbc;

import java.io.*;
import java.math.BigDecimal;

import java.sql.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.server.Unreferenced;


// TBD: WARNING This file contains a hack for InputStream class...
// InputStream is not serializable, of course !
// The right way would be to encapsulate InputStream in a RMI remote object
// (hope I'll find time to do that)

/**
 * <P>A ResultSet provides access to a table of data generated by
 * executing a Statement. The table rows are retrieved in
 * sequence. Within a row its column values can be accessed in any
 * order.
 * 
 * <P>A ResultSet maintains a cursor pointing to its current row of
 * data.  Initially the cursor is positioned before the first row.
 * The 'next' method moves the cursor to the next row.
 *
 * <P>The getXXX methods retrieve column values for the current
 * row.  You can retrieve values either using the index number of the
 * column, or by using the name of the column.  In general using the 
 * column index will be more efficient.  Columns are numbered from 1.
 *
 * <P>For maximum portability, ResultSet columns within each row should be
 * read in left-to-right order and each column should be read only once.
 *
 * <P>For the getXXX methods, the JDBC driver attempts to convert the
 * underlying data to the specified Java type and returns a suitable
 * Java value.  See the JDBC specification for allowable mappings
 * from SQL types to Java types with the ResultSet.getXXX methods.
 *
 * <P>Column names used as input to getXXX methods are case insensitive.
 * When performing a getXXX using a column name, if several columns have
 * the same name, then the value of the first matching column will be
 * returned.
 *
 * <P>A ResultSet is automatically closed by the Statement that
 * generated it when that Statement is closed, re-executed, or is used
 * to retrieve the next result from a sequence of multiple results.
 * 
 * <P>The number, types and properties of a ResultSet's columns are
 * provided by the ResulSetMetaData object returned by the getMetaData
 * method.
 *
 * @see Statement#executeQuery 
 * @see Statement#getResultSet 
 * @see ResultSetMetaData 
 */

public class RJResultSetServer
extends java.rmi.server.UnicastRemoteObject
implements RJResultSetInterface, Unreferenced {

  java.sql.ResultSet jdbcResultSet_;

  public RJResultSetServer(java.sql.ResultSet s)
  throws java.rmi.RemoteException {
    super();
    jdbcResultSet_ = s;
  }

  public void unreferenced() { Runtime.getRuntime().gc(); }

  protected void finalize() throws Exception {
    if(jdbcResultSet_ != null) jdbcResultSet_.close();
  }

  /**
   * A ResultSet is initially positioned before its first row; the
   * first call to next makes the first row the current row; the
   * second call makes the second row the current row, etc. 
   *
   * <P>If an input stream from the previous row is open, it is
   * implicitly closed. The ResultSet's warning chain is cleared
   * when a new row is read.
   *
   * @return true if the new current row is valid; false if there
   * are no more rows 
   */
  public boolean next() throws RemoteException, SQLException {
    return jdbcResultSet_.next();
  }


  /**
   * In some cases, it is desirable to immediately release a
   * ResultSet's database and JDBC resources instead of waiting for
   * this to happen when it is automatically closed; the close
   * method provides this immediate release.
   *
   * <P><B>Note:</B> A ResultSet is automatically closed by the
   * Statement that generated it when that Statement is closed,
   * re-executed, or is used to retrieve the next result from a
   * sequence of multiple results. A ResultSet is also automatically
   * closed when it is garbage collected.  
   */
  public void close() throws RemoteException, SQLException {
    if(jdbcResultSet_ != null) jdbcResultSet_.close();
  }

  /**
   * A column may have the value of SQL NULL; wasNull reports whether
   * the last column read had this special value.
   * Note that you must first call getXXX on a column to try to read
   * its value and then call wasNull() to find if the value was
   * the SQL NULL.
   *
   * @return true if last column read was SQL NULL
   */
  public boolean wasNull() throws RemoteException, SQLException {
    return jdbcResultSet_.wasNull();
  }
    
  //======================================================================
  // Methods for accessing results by column index
  //======================================================================

  /**
   * Get the value of a column in the current row as a Java String.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public String getString(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getString(columnIndex);
  }

  /**
   * Get the value of a column in the current row as a Java boolean.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is false
   */
  public boolean getBoolean(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getBoolean(columnIndex);
  }

    /**
     * Get the value of a column in the current row as a Java byte.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return the column value; if the value is SQL NULL, the result is 0
     */
  public byte getByte(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getByte(columnIndex);
  }

    /**
     * Get the value of a column in the current row as a Java short.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return the column value; if the value is SQL NULL, the result is 0
     */
  public short getShort(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getShort(columnIndex);
  }

  /**
   * Get the value of a column in the current row as a Java int.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is 0
   */
  public int getInt(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getInt(columnIndex);
  }

  /**
   * Get the value of a column in the current row as a Java long.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is 0
   */
  public long getLong(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getLong(columnIndex);
  }

  /**
   * Get the value of a column in the current row as a Java float.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is 0
   */
  public float getFloat(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getFloat(columnIndex);
  }

  /**
   * Get the value of a column in the current row as a Java double.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is 0
   */
  public double getDouble(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getDouble(columnIndex);
  }

  /**
   * Get the value of a column in the current row as a java.lang.BigDecimal object.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param scale the number of digits to the right of the decimal
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public BigDecimal getBigDecimal(int columnIndex, int scale)
  throws RemoteException, SQLException {
    return jdbcResultSet_.getBigDecimal(columnIndex, scale);
  }

  /**
   * Get the value of a column in the current row as a Java byte array.
   * The bytes represent the raw values returned by the driver.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL, the result is null
   */
  public byte[] getBytes(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getBytes(columnIndex);
  }

    /**
     * Get the value of a column in the current row as a java.sql.Date object.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public java.sql.Date getDate(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getDate(columnIndex);
  }

    /**
     * Get the value of a column in the current row as a java.sql.Time object.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public java.sql.Time getTime(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getTime(columnIndex);
  }

    /**
     * Get the value of a column in the current row as a java.sql.Timestamp object.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public java.sql.Timestamp getTimestamp(int columnIndex) throws RemoteException, SQLException {
    return jdbcResultSet_.getTimestamp(columnIndex);
  }

    /**
     * A column value can be retrieved as a stream of ASCII characters 
     * and then read in chunks from the stream.  This method is particularly
     * suitable for retrieving large LONGVARCHAR values.  The JDBC driver will
     * do any necessary conversion from the database format into ASCII.
     *
     * <P><B>Note:</B> All the data in the returned stream must be
     * read prior to getting the value of any other column. The next
     * call to a get method implicitly closes the stream. . Also, a
     * stream may return 0 for available() whether there is data
     * available or not.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return a Java input stream that delivers the database column value
     * as a stream of one byte ASCII characters.  If the value is SQL NULL
     * then the result is null.  
     */
//  public java.io.InputStream getAsciiStream(int columnIndex)
// TBD: This is a hack: InputStream is not serializable !
  public byte[] getAsciiStream(int columnIndex)
  throws RemoteException, SQLException {
    try {
      // I read the whole InputStream into a StringBuffer, then send the
      // StringBuffer's bytes to the client !!  Awful, isn't it :(
      // The right way is to "RMIze" InputStream, or what ??
      InputStream s = jdbcResultSet_.getAsciiStream(columnIndex);
      if(s == null) return null;
      StringBuffer buf = new StringBuffer(); 
      BufferedReader r = new BufferedReader(new InputStreamReader(s));
      char cbuf[] = new char[256];
      int nb;
      while((nb = r.read(cbuf, 0, 255)) >= 0) {
        if(nb > 0) buf.append(cbuf, 0, nb);
      }
      r.close();
      return buf.toString().getBytes();
    } catch(IOException e) {
      throw new java.rmi.RemoteException(
       "RJResultSetServer::getAsciiStream()", e);
    }
  }

    /**
     * A column value can be retrieved as a stream of Unicode characters 
     * and then read in chunks from the stream.  This method is particularly
     * suitable for retrieving large LONGVARCHAR values.  The JDBC driver will
     * do any necessary conversion from the database format into Unicode.
     *
     * <P><B>Note:</B> All the data in the returned stream must be
     * read prior to getting the value of any other column. The next
     * call to a get method implicitly closes the stream. . Also, a
     * stream may return 0 for available() whether there is data
     * available or not.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return a Java input stream that delivers the database column value
     * as a stream of two byte Unicode characters.  If the value is SQL NULL
     * then the result is null.  
     */
//  public java.io.InputStream getUnicodeStream(int columnIndex)
// TBD: This is a hack: InputStream is not serializable !
  public byte[] getUnicodeStream(int columnIndex)
  throws RemoteException, SQLException {
    try {
      // I read the whole InputStream into a StringBuffer, then send the
      // StringBuffer's bytes to the client !!  Awful, isn't it :(
      // The right way is to "RMIze" InputStream, or what ??
      InputStream s = jdbcResultSet_.getUnicodeStream(columnIndex);
      if(s == null) return null;
      ByteArrayOutputStream bos = new ByteArrayOutputStream();
      byte buf[] = new byte[256];
      int br;
      while((br = s.read(buf)) >= 0){
        if (br > 0) bos.write(buf, 0, br);
      }
      s.close();
      return bos.toByteArray();
    } catch(IOException e) {
      throw new java.rmi.RemoteException(
       "RJResultSetServer::getUnicodeStream()", e);
    }
  }

    /**
     * A column value can be retrieved as a stream of uninterpreted bytes 
     * and then read in chunks from the stream.  This method is particularly
     * suitable for retrieving large LONGVARBINARY values.
     *
     * <P><B>Note:</B> All the data in the returned stream must be
     * read prior to getting the value of any other column. The next
     * call to a get method implicitly closes the stream. Also, a
     * stream may return 0 for available() whether there is data
     * available or not.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return a Java input stream that delivers the database column value
     * as a stream of uninterpreted bytes.  If the value is SQL NULL
     * then the result is null.  
     */
//  public java.io.InputStream getBinaryStream(int columnIndex)
// TBD: This is a hack: InputStream is not serializable !
  public byte[] getBinaryStream(int columnIndex)
  throws RemoteException, SQLException {
    try {
      // I read the whole InputStream into a StringBuffer, then send the
      // StringBuffer's bytes to the client !!  Awful, isn't it :(
      // The right way is to "RMIze" InputStream, or what ??
      InputStream s = jdbcResultSet_.getBinaryStream(columnIndex);
      if(s == null) return null;
      StringBuffer buf = new StringBuffer(); 
      BufferedReader r = new BufferedReader(new InputStreamReader(s));
      char cbuf[] = new char[256];
      int nb;
      while((nb = r.read(cbuf, 0, 255)) >= 0) {
        if(nb > 0) buf.append(cbuf, 0, nb);
      }
      r.close();
      return buf.toString().getBytes();
    } catch(IOException e) {
      throw new java.rmi.RemoteException(
       "RJResultSetServer::getBinaryStream()", e);
    }
  }


    //======================================================================
    // Methods for accessing results by column name
    //======================================================================

    /**
     * Get the value of a column in the current row as a Java String.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public String getString(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getString(columnName);
  }

    /**
     * Get the value of a column in the current row as a Java boolean.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is false
     */
  public boolean getBoolean(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getBoolean(columnName);
  }

    /**
     * Get the value of a column in the current row as a Java byte.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is 0
     */
  public byte getByte(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getByte(columnName);
  }

    /**
     * Get the value of a column in the current row as a Java short.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is 0
     */
  public short getShort(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getShort(columnName);
  }

  /**
   * Get the value of a column in the current row as a Java int.
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if the value is SQL NULL, the result is 0
   */
  public int getInt(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getInt(columnName);
  }

    /**
     * Get the value of a column in the current row as a Java long.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is 0
     */
  public long getLong(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getLong(columnName);
  }

    /**
     * Get the value of a column in the current row as a Java float.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is 0
     */
  public float getFloat(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getFloat(columnName);
  }

    /**
     * Get the value of a column in the current row as a Java double.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is 0
     */
  public double getDouble(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getDouble(columnName);
  }

    /**
     * Get the value of a column in the current row as a java.lang.BigDecimal object.
     *
     * @param columnName is the SQL name of the column
     * @param scale the number of digits to the right of the decimal
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public BigDecimal getBigDecimal(String columnName, int scale)
  throws RemoteException, SQLException {
    return jdbcResultSet_.getBigDecimal(columnName, scale);
  }

    /**
     * Get the value of a column in the current row as a Java byte array.
     * The bytes represent the raw values returned by the driver.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public byte[] getBytes(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getBytes(columnName);
  }

    /**
     * Get the value of a column in the current row as a java.sql.Date object.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public java.sql.Date getDate(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getDate(columnName);
  }

    /**
     * Get the value of a column in the current row as a java.sql.Time object.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public java.sql.Time getTime(String columnName) throws RemoteException, SQLException {
    return jdbcResultSet_.getTime(columnName);
  }

    /**
     * Get the value of a column in the current row as a java.sql.Timestamp object.
     *
     * @param columnName is the SQL name of the column
     * @return the column value; if the value is SQL NULL, the result is null
     */
  public java.sql.Timestamp getTimestamp(String columnName)
  throws RemoteException, SQLException {
    return jdbcResultSet_.getTimestamp(columnName);
  }

    /**
     * A column value can be retrieved as a stream of ASCII characters 
     * and then read in chunks from the stream.  This method is particularly
     * suitable for retrieving large LONGVARCHAR values.  The JDBC driver will
     * do any necessary conversion from the database format into ASCII.
     *
     * <P><B>Note:</B> All the data in the returned stream must
     * be read prior to getting the value of any other column. The
     * next call to a get method implicitly closes the stream.
     *
     * @param columnName is the SQL name of the column
     * @return a Java input stream that delivers the database column value
     * as a stream of one byte ASCII characters.  If the value is SQL NULL
     * then the result is null.
     */
//  public java.io.InputStream getAsciiStream(String columnName)
// TBD: This is a hack: InputStream is not serializable !
  public byte[] getAsciiStream(String columnName)
  throws RemoteException, SQLException {
    try {
      // I read the whole InputStream into a StringBuffer, then send the
      // StringBuffer's bytes to the client !!  Awful, isn't it :(
      // The right way is to "RMIze" InputStream, or what ??
      InputStream s = jdbcResultSet_.getAsciiStream(columnName);
      if(s == null) return null;
      StringBuffer buf = new StringBuffer(); 
      BufferedReader r = new BufferedReader(new InputStreamReader(s));
      char cbuf[] = new char[256];
      int nb;
      while((nb = r.read(cbuf, 0, 255)) >= 0) {
        if(nb > 0) buf.append(cbuf, 0, nb);
      }
      r.close();
      return buf.toString().getBytes();
    } catch(IOException e) {
      throw new java.rmi.RemoteException(
       "RJResultSetServer::getAsciiStream()", e);
    }
  }

    /**
     * A column value can be retrieved as a stream of Unicode characters 
     * and then read in chunks from the stream.  This method is particularly
     * suitable for retrieving large LONGVARCHAR values.  The JDBC driver will
     * do any necessary conversion from the database format into Unicode.
     *
     * <P><B>Note:</B> All the data in the returned stream must
     * be read prior to getting the value of any other column. The
     * next call to a get method implicitly closes the stream.
     *
     * @param columnName is the SQL name of the column
     * @return a Java input stream that delivers the database column value
     * as a stream of two byte Unicode characters.  If the value is SQL NULL
     * then the result is null.
     */
//  public java.io.InputStream getUnicodeStream(String columnName)
// TBD: This is a hack: InputStream is not serializable !
  public byte[] getUnicodeStream(String columnName)
  throws RemoteException, SQLException {
    try {
      // I read the whole InputStream into a StringBuffer, then send the
      // StringBuffer's bytes to the client !!  Awful, isn't it :(
      // The right way is to "RMIze" InputStream, or what ??
      InputStream s = jdbcResultSet_.getUnicodeStream(columnName);
      if(s == null) return null;
      StringBuffer buf = new StringBuffer(); 
      BufferedReader r = new BufferedReader(new InputStreamReader(s));
      char cbuf[] = new char[256];
      int nb;
      while((nb = r.read(cbuf, 0, 255)) >= 0) {
        if(nb > 0) buf.append(cbuf, 0, nb);
      }
      r.close();
      return buf.toString().getBytes();
    } catch(IOException e) {
      throw new java.rmi.RemoteException(
       "RJResultSetServer::getUnicodeStream()", e);
    }
  }

    /**
     * A column value can be retrieved as a stream of uninterpreted bytes 
     * and then read in chunks from the stream.  This method is particularly
     * suitable for retrieving large LONGVARBINARY values.
     *
     * <P><B>Note:</B> All the data in the returned stream must
     * be read prior to getting the value of any other column. The
     * next call to a get method implicitly closes the stream.
     *
     * @param columnName is the SQL name of the column
     * @return a Java input stream that delivers the database column value
     * as a stream of uninterpreted bytes.  If the value is SQL NULL
     * then the result is null.
     */
//  public java.io.InputStream getBinaryStream(String columnName)
// TBD: This is a hack: InputStream is not serializable !
  public byte[] getBinaryStream(String columnName)
  throws RemoteException, SQLException {
    try {
      // I read the whole InputStream into a StringBuffer, then send the
      // StringBuffer's bytes to the client !!  Awful, isn't it :(
      // The right way is to "RMIze" InputStream, or what ??
      InputStream s = jdbcResultSet_.getBinaryStream(columnName);
      if(s == null) return null;
      ByteArrayOutputStream bos = new ByteArrayOutputStream();
      byte buf[] = new byte[256];
      int br;
      while((br = s.read(buf)) >= 0){
        if (br > 0) bos.write(buf, 0, br);
      }
      s.close();
      return bos.toByteArray();
    } catch(IOException e) {
      throw new java.rmi.RemoteException(
       "RJResultSetServer::getBinaryStream()", e);
    }
  }


    //=====================================================================
    // Advanced features:
    //=====================================================================

    /**
     * <p>The first warning reported by calls on this ResultSet is
     * returned. Subsequent ResultSet warnings will be chained to this
     * SQLWarning.
     *
     * <P>The warning chain is automatically cleared each time a new
     * row is read.
     *
     * <P><B>Note:</B> This warning chain only covers warnings caused
     * by ResultSet methods.  Any warning caused by statement methods
     * (such as reading OUT parameters) will be chained on the
     * Statement object. 
     *
     * @return the first SQLWarning or null 
     */
  public SQLWarning getWarnings() throws RemoteException, SQLException {
    return jdbcResultSet_.getWarnings();
  }

    /**
     * After this call getWarnings returns null until a new warning is
     * reported for this ResultSet.  
     */
  public void clearWarnings() throws RemoteException, SQLException {
    jdbcResultSet_.clearWarnings();
  }

    /**
     * Get the name of the SQL cursor used by this ResultSet.
     *
     * <P>In SQL, a result table is retrieved through a cursor that is
     * named. The current row of a result can be updated or deleted
     * using a positioned update/delete statement that references the
     * cursor name. 
     * 
     * <P>JDBC supports this SQL feature by providing the name of the
     * SQL cursor used by a ResultSet. The current row of a ResultSet
     * is also the current row of this SQL cursor.
     *
     * <P><B>Note:</B> If positioned update is not supported a
     * java.rmi.RemoteException is thrown
     *
     * @return the ResultSet's SQL cursor name
     */
  public String getCursorName() throws RemoteException, SQLException {
    return jdbcResultSet_.getCursorName();
  }

    /**
     * The number, types and properties of a ResultSet's columns
     * are provided by the getMetaData method.
     *
     * @return the description of a ResultSet's columns
     */
  public RJResultSetMetaDataInterface getMetaData()
  throws RemoteException, SQLException {
    return new RJResultSetMetaDataServer(jdbcResultSet_.getMetaData());
  }

    /**
     * <p>Get the value of a column in the current row as a Java object.
     *
     * <p>This method will return the value of the given column as a
     * Java object.  The type of the Java object will be the default
     * Java Object type corresponding to the column's SQL type,
     * following the mapping specified in the JDBC spec.
     *
     * <p>This method may also be used to read datatabase specific abstract
     * data types.
     *
     * @param columnIndex the first column is 1, the second is 2, ...
     * @return A java.lang.Object holding the column value.  
     */
  public Object getObject(int columnIndex)
  throws RemoteException, SQLException {
    return jdbcResultSet_.getObject(columnIndex);
  }

    /**
     * <p>Get the value of a column in the current row as a Java object.
     *
     * <p>This method will return the value of the given column as a
     * Java object.  The type of the Java object will be the default
     * Java Object type corresponding to the column's SQL type,
     * following the mapping specified in the JDBC spec.
     *
     * <p>This method may also be used to read datatabase specific abstract
     * data types.
     *
     * @param columnName is the SQL name of the column
     * @return A java.lang.Object holding the column value.  
     */
  public Object getObject(String columnName)
  throws RemoteException, SQLException {
    return jdbcResultSet_.getObject(columnName);
  }

    //----------------------------------------------------------------

    /**
     * Map a Resultset column name to a ResultSet column index.
     *
     * @param columnName the name of the column
     * @return the column index
     */

  public int findColumn(String columnName)
  throws RemoteException, SQLException {
    return jdbcResultSet_.findColumn(columnName);
  }

};

